# -*- coding: utf-8 -*-
#
# Configuration file for the Sphinx documentation builder.
#
# This file does only contain a selection of the most common options. For a
# full list see the documentation:
# http://www.sphinx-doc.org/en/master/config

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys

from datetime import date

from ms_graph_exporter import __version__

sys.path.insert(0, os.path.abspath(".."))

# -- Project information -----------------------------------------------------

project = "PyKV"
copyright = "2020, UNDP"
author = "Oleksiy Kuzmenko <oleksiy.kuzmenko@undp.org>"

if date.today().year != 2020:
    copyright = "2020-{}, UNDP".format(date.today().year)
else:
    copyright = "2019, UNDP"

# The short X.Y version
version = __version__
# The full version, including alpha/beta/rc tags
release = version


# -- General configuration ---------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.intersphinx",
    "sphinx.ext.coverage",
    "sphinx.ext.viewcode",
    "sphinx.ext.githubpages",
    "sphinx.ext.napoleon",
    "sphinx_autodoc_typehints",
    "recommonmark",
    "celery.contrib.sphinx",
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
source_suffix = [".rst", ".md"]

# The master toctree document.
master_doc = "index"

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = None

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "sphinx_rtd_theme"

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
html_theme_options = {
    # 'canonical_url': '',
    # 'analytics_id': 'UA-XXXXXXX-1',
    # 'logo_only': False,
    # "display_version": False,
    # 'prev_next_buttons_location': 'bottom',
    # 'style_external_links': False,
    # 'vcs_pageview_mode': '',
    # 'style_nav_header_background': 'white',
    # # Toc options
    # 'collapse_navigation': True,
    # 'sticky_navigation': True,
    # 'navigation_depth': 4,
    # 'includehidden': True,
    # 'titles_only': False
}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

# Custom sidebar templates, must be a dictionary that maps document names
# to template names.
#
# The default sidebars (for documents that don't match any pattern) are
# defined by theme itself.  Builtin themes are using these templates by
# default: ``['localtoc.html', 'relations.html', 'sourcelink.html',
# 'searchbox.html']``.
#
# html_sidebars = {}


# -- Options for HTMLHelp output ---------------------------------------------

# If true, links to the reST sources are added to the pages.
html_show_sourcelink = False

# Output file base name for HTML help builder.
htmlhelp_basename = "PyKV"


# -- Extension configuration -------------------------------------------------

# -- Options for sphinx-napoleon extension -----------------------------------

# True to parse Google style docstrings. False to disable support for Google
# style docstrings. Defaults to True.
napoleon_google_docstring = False

# True to list __init___ docstrings separately from the class docstring. False
# to fall back to Sphinx’s default behavior, which considers the __init___
# docstring as part of the class documentation. Defaults to False.
napoleon_include_init_with_doc = True

# True to include private members (like _membername) with docstrings in the
# documentation. False to fall back to Sphinx’s default behavior.
# Defaults to False.
napoleon_include_private_with_doc = True

# True to include special members (like __membername__) with docstrings in the
# documentation. False to fall back to Sphinx’s default behavior.
# Defaults to True.
napoleon_include_special_with_doc = True

# True to use a :param: role for each function parameter. False to use a single
# :parameters: role for all the parameters.
# Defaults to True.
napoleon_use_param = True

# True to use the :ivar: role for instance variables. False to use the
# .. attribute:: directive instead. Defaults to False.
napoleon_use_ivar = True

# True to use the :rtype: role for the return type. False to output the
# return type inline with the description.
# Defaults to True.
napoleon_use_rtype = True

# -- Options for intersphinx extension ---------------------------------------

intersphinx_mapping = {
    "python": ("https://docs.python.org/", None),
}



"""Keyvaults controller module."""
from typing import Any, Dict, List

from cement import Controller, ex


class Keyvaults(Controller):
    """ Class implementing controller for ``keyvaults`` namespace."""

    class Meta:
        """Controller meta-data."""

        label: str = "keyvaults"
        stacked_on: str = "base"
        stacked_type: str = "nested"
        help: str = "Operations with key vaults"  # noqa: A003

    @ex(help="list all Azure Key Vaults in use by the app")
    def show(self) -> None:
        """List all Key Vaults from config."""
        # get keyvaults and their short names from config
        keyvaults: Dict[str, Any] = self.app.config.get("azkv", "keyvaults")

        output_data: Dict[str, List[Any]] = {"keyvaults": []}

        for vault in keyvaults.keys():
            output_data["keyvaults"].append(
                {"name": vault, "url": keyvaults[vault]["url"]}
            )

        self.app.render(output_data, "keyvaults_list.j2")



# -*- coding: utf-8 -*-
"""Module defines common test fixtures."""
from logging import getLogger

from cement import fs

import pytest


@pytest.fixture(scope="session")
def logger():
    """Provide logger instance."""
    logger = getLogger(__name__)

    return logger


@pytest.fixture(scope="function")
def tmp(request):
    """Provide temporary directory.

    Create a `Tmp` object that generates a unique temporary directory, and file
    for each test function that requires it.
    """
    t = fs.Tmp()
    yield t
    t.remove()



"""Base app controller module."""
from cement import Controller
from cement.utils.version import get_version_banner

from ..core.version import get_version

VERSION_BANNER = """
AzKV %s
%s
""" % (
    get_version(),
    get_version_banner(),
)


class Base(Controller):
    """ Class implementing base app controller."""

    class Meta:
        """Controller meta-data."""

        label = "base"

        description = "CLI client for the Azure Key Vault data plane."
        epilog = "Usage: azkv secrets search <secret_name>"

        arguments = [
            # add a version banner
            (["-v", "--version"], {"action": "version", "version": VERSION_BANNER}),
        ]



# -*- coding: utf-8 -*-
"""Framework hooks module."""
from typing import Any, Dict, Optional, Union

from azure.identity import (
    EnvironmentCredential,
    ManagedIdentityCredential,
)

from cement import App


def extend_vault_creds(app: App) -> None:
    """Extend app with azure credentials for each vault.

    Obtains Azure identity either from Environment Variables or Managed Identity.

    Parameters
    ----------
    app
        Cement Framework application object.
    """
    app.log.info("Extending app object with Azure Key Vault credentials")

    common_creds_config: Dict[str, str] = app.config.get("azkv", "credentials")
    if common_creds_config:
        common_type: Optional[str] = common_creds_config.get("type")
        common_client_id: Optional[str] = common_creds_config.get("client_id")
    else:
        common_type = "EnvironmentVariables"
        common_client_id = None

    keyvaults: Dict[str, Any] = app.config.get("azkv", "keyvaults")

    vault_creds: Dict[str, Union[EnvironmentCredential, ManagedIdentityCredential]] = {}

    creds_from_env = EnvironmentCredential()
    creds_from_mi = ManagedIdentityCredential()

    for vault, config in keyvaults.items():
        creds_config = config.get("credentials", None)

        if creds_config:
            creds_type = creds_config.get("type", common_type)
            creds_client_id = creds_config.get("client_id", common_client_id)
        else:
            creds_type = common_type
            creds_client_id = common_client_id

        app.log.info(
            "Vault '{}' would be queried with credentials from '{}'".format(  # noqa: G001, E501
                vault, creds_type
            )
        )

        if creds_type == "EnvironmentVariables":
            vault_creds[vault] = creds_from_env

        elif creds_type == "SystemManagedIdentity":
            vault_creds[vault] = creds_from_mi

        elif creds_type == "UserManagedIdentity":
            vault_creds[vault] = ManagedIdentityCredential(client_id=creds_client_id)

            app.log.info("  client_id={}".format(creds_client_id))  # noqa: G001

            if creds_client_id is None:
                app.log.warning(
                    "  no 'client_id' defied, changed to credentials from SystemManagedIdentity"  # noqa: E501
                )

        else:
            # fmt: off
            app.log.warning(
                "Unknown value '{}' in credentials type, assume 'EnvironmentVariables'".format(creds_type)  # noqa: G001, E501
            )
            # fmt: on

            vault_creds[vault] = creds_from_env

    app.extend("vault_creds", vault_creds)



# -*- coding: utf-8 -*-
"""Log handler module."""
from cement.ext.ext_colorlog import ColorLogHandler


class AzKVLogHandler(ColorLogHandler):
    """Class implementing log handler with custom format.

    This class is a sub-class of :class:`cement.ext.ext_colorlog.ColorLogHandler`,
    and it changes log format for console and file outputs.

    """

    class Meta:
        """Handler meta-data."""

        label = "colorlog_custom_format"

        console_format = "%(asctime)-15s %(levelname)-8s %(namespace)s : %(message)s"



"""Module defines app test cases."""
from azkv.main import AzKVTest


def test_azkv():
    """Test azkv without any subcommands or arguments."""
    with AzKVTest() as app:
        app.run()
        assert app.exit_code == 0  # noqa: S101


def test_azkv_debug():
    """Test that debug mode is functional."""
    argv = ["--debug"]
    with AzKVTest(argv=argv) as app:
        app.run()
        assert app.debug is True  # noqa: S101



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['azkv', 'azkv.controllers', 'azkv.core', 'azkv.ext', 'azkv.templates']

package_data = \
{'': ['*']}

install_requires = \
['azure-common>=1.1.25,<2.0.0',
 'azure-identity>=1.3.1,<1.4.0',
 'azure-keyvault-secrets>=4.2.0,<5.0.0',
 'cement>=3.0.4,<4.0.0',
 'colorlog>=4.2.1,<5.0.0',
 'jinja2>=2.11.2,<3.0.0',
 'pyyaml>=5.3.1,<6.0.0',
 'typing>=3.7.4,<4.0.0']

extras_require = \
{'code-format': ['black>=19.10b0,<20.0', 'blacken-docs>=1.7,<2.0'],
 'code-lint': ['flake8>=3.8,<4.0',
               'flake8-import-order>=0.18,<0.19',
               'flake8-bandit>=2.1,<3.0',
               'flake8-blind-except>=0.1,<0.2',
               'flake8-bugbear>=20.1,<21.0',
               'flake8-builtins>=1.5,<2.0',
               'flake8-docstrings>=1.5,<2.0',
               'flake8-logging-format>=0.6,<0.7',
               'flake8-mypy>=17.8,<18.0',
               'pep8-naming>=0.8,<0.9',
               'pygments>=2.6,<3.0'],
 'docs': ['recommonmark>=0.6.0,<0.7.0',
          'sphinx>=3.1,<4.0',
          'sphinx-rtd-theme>=0.5,<0.6',
          'sphinx-autodoc-typehints>=1.11,<2.0'],
 'test': ['pytest>=6.0,<7.0',
          'pytest-benchmark[aspect]>=3.2,<4.0',
          'pytest-cov>=2.10,<3.0',
          'pytest-instafail>=0.4,<0.5',
          'pytest-lazy-fixture>=0.6,<0.7',
          'pytest-random-order>=1.0,<2.0']}

entry_points = \
{'console_scripts': ['azkv = azkv.main:main']}

setup_kwargs = {
    'name': 'azkv',
    'version': '0.1.0rc3',
    'description': 'CLI client for the Azure Key Vault data plane',
    'long_description': '# AzKV\n\n[![Python 3.6+](https://img.shields.io/badge/Python-3.6+-blue.svg)][PythonRef] [![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)][BlackRef] [![License: MIT](https://img.shields.io/badge/License-MIT-blue.svg)][MITRef]\n\n[PythonRef]: https://docs.python.org/3.6/\n[BlackRef]: https://github.com/ambv/black\n[MITRef]: https://opensource.org/licenses/MIT\n\n`azkv` is a CLI client for the Azure Key Vault data plane.\n\n## Getting Started\n\n### Installing\n\nTBD\n\n### Requirements\n\n* Python >= 3.6\n\n## Built using\n\n* [Cement][CementRef] - CLI application framework\n\n[CementRef]: https://builtoncement.com/\n\n## Versioning\n\nWe use [Semantic Versioning Specification][SemVer] as a version numbering convention.\n\n[SemVer]: http://semver.org/\n\n## Release History\n\nFor the available versions, see the [tags on this repository][RepoTags]. Specific changes for each version are documented in [CHANGELOG.md][ChangelogRef].\n\nAlso, conventions for `git commit` messages are documented in [CONTRIBUTING.md][ContribRef].\n\n[RepoTags]: https://github.com/undp/azkv/tags\n[ChangelogRef]: CHANGELOG.md\n[ContribRef]: CONTRIBUTING.md\n\n## Authors\n\n* **Oleksiy Kuzmenko** - [OK-UNDP@GitHub][OK-UNDP@GitHub] - *Initial design and implementation*\n\n[OK-UNDP@GitHub]: https://github.com/OK-UNDP\n\n## Acknowledgments\n\n* Hat tip to all individuals shaping design of this project by sharing their knowledge in articles, blogs and forums.\n\n## License\n\nUnless otherwise stated, all authors (see commit logs) release their work under the [MIT License][MITRef]. See [LICENSE.md][LicenseRef] for details.\n\n[LicenseRef]: LICENSE.md\n\n## Contributing\n\nThere are plenty of ways you could contribute to this project. Feel free to:\n\n* submit bug reports and feature requests\n* outline, fix and expand documentation\n* peer-review bug reports and pull requests\n* implement new features or fix bugs\n\nSee [CONTRIBUTING.md][ContribRef] for details on code formatting, linting and testing frameworks used by this project.\n',
    'author': 'Oleksiy Kuzmenko',
    'author_email': 'oleksiy.kuzmenko@undp.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/undp/azkv',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



"""Main app module."""
from cement import App, TestApp, init_defaults
from cement.core.exc import CaughtSignal

from .controllers.base import Base
from .controllers.keyvaults import Keyvaults
from .controllers.secrets import Secrets
from .core.exc import AzKVError
from .core.hooks import extend_vault_creds
from .core.log import AzKVLogHandler

# configuration defaults
CONFIG = init_defaults("azkv", "azkv.credentials", "azkv.keyvaults")
CONFIG["azkv"]["credentials"] = {"type": "EnvironmentVariables"}
CONFIG["azkv"]["keyvaults"] = []


class AzKV(App):
    """AzKV primary application."""

    class Meta:
        """Application meta-data."""

        label = "azkv"

        # configuration defaults
        config_defaults = CONFIG

        # call sys.exit() on close
        exit_on_close = True

        # register functions to hooks
        hooks = [
            ("post_setup", extend_vault_creds),
        ]

        # load additional framework extensions
        extensions = [
            "colorlog",
            "jinja2",
            "yaml",
        ]

        # configuration handler
        config_handler = "yaml"

        # configuration file suffix
        config_file_suffix = ".yaml"

        # set log handler
        log_handler = "colorlog_custom_format"

        # set the output handler
        output_handler = "jinja2"

        # register handlers
        handlers = [Base, AzKVLogHandler, Keyvaults, Secrets]


class AzKVTest(TestApp, AzKV):
    """A sub-class of AzKV that is better suited for testing."""

    class Meta:
        """Test application meta-data."""

        label = "azkv"


def main():
    """App entry point."""
    with AzKV() as app:
        try:
            app.run()

        except AssertionError as e:
            print("AssertionError > %s" % e.args[0])
            app.exit_code = 1

            if app.debug is True:
                import traceback

                traceback.print_exc()

        except AzKVError as e:
            print("AzKVError > %s" % e.args[0])
            app.exit_code = 1

            if app.debug is True:
                import traceback

                traceback.print_exc()

        except CaughtSignal as e:
            # Default Cement signals are SIGINT and SIGTERM, exit 0 (non-error)
            print("\n%s" % e)
            app.exit_code = 0


if __name__ == "__main__":
    main()
